import { useRef, useEffect } from 'react';
import {
  select,
  forceSimulation,
  forceCenter,
  forceManyBody,
  forceCollide,
  forceLink,
  drag
} from 'd3';
import { MANY_BODY_STRENGTH, MIN_NODE_DISTANCE, nodeColorMap } from './useData';
import { colors } from './colors';

export const ForceNetwork = ({nodes, links, innerWidth, innerHeight}) => {
  //console.log('nodes: ', nodes)
  //console.log('links: ', links)
  const gRef = useRef();
  useEffect(() => {
    const g = select(gRef.current);
    
    const centerX = innerWidth / 2;
    const centerY = innerHeight / 2;
    
    const simulation = forceSimulation(nodes)
    	.force('center', forceCenter(centerX, centerY))
    	.force('charge', forceManyBody().strength(MANY_BODY_STRENGTH))
    	.force('collide', forceCollide(MIN_NODE_DISTANCE))
    	.force('link', forceLink(links).id(d => d.id).distance(link => link.distance))
    	.alphaTarget(0.1)
    	.alphaMin(0.15);

    const lines = g.selectAll('line').data(links)
    	.enter().append('line')
    		.attr('stroke', link => colors[nodeColorMap.get(link.source.id)][4]); // change color index from 0~4 for different intensity versions
    
    const circles = g.selectAll('circle').data(nodes)
    	.enter().append('circle')
    		.attr('fill', node => colors[nodeColorMap.get(node.id)][2]) // change color index from 0~4 for different intensity versions
    		.attr('r', node => node.size);
    
    const text = g.selectAll('text').data(nodes)
    	.enter().append('text')
    		.attr('text-anchor', 'middle')
    		.attr('alignment-baseline', 'middle')
    		.style('pointer-events', 'none')
    		.text(node => node.id.startsWith('anonymous') ? '' : node.id);
    
    simulation.on('tick', () => {
      //console.log('current force', simulation.alpha());
      lines
      	.attr('x1', link => link.source.x)
      	.attr('y1', link => link.source.y)
      	.attr('x2', link => link.target.x)
      	.attr('y2', link => link.target.y)
      circles
      	.attr('cx', node => node.x)
      	.attr('cy', node => node.y);
      text
      	.attr('x', node => node.x)
      	.attr('y', node => node.y);
    });
    
    // drag interaction
    const dragInteraction = drag().on('drag', (event, node) => {
      node.fx = event.x;
      node.fy = event.y;
      simulation.alpha(1).restart();
    });
    circles.call(dragInteraction)
    
  }, []);

  return <g ref={gRef} />;
};
